<?php

if (!defined('ABSPATH')) exit;


if (isset($_POST['options_name'])) {

	if (!wp_verify_nonce($_POST[ALR_Setting::NONCE_NAME], ALR_Setting::NONCE_ACTION)) wp_die();

	$options_name = isset($_POST['options_name']) ? sanitize_text_field($_POST['options_name']) : '';
	unset($_POST['options_name']);

	if (ALR_Setting::OPTIONS_NAME['settings'] !== $options_name) {

		wp_die();
	}

	if (!empty($_POST['asp_string'])) {
		foreach ($_POST['asp_string'] as $key_a => $link_a) {
			if (empty($link_a)) {
				$error = __('リンク文字列を入力してください。', 'affiliate-link-redirector');
				break;
			}
			foreach ($_POST['asp_string'] as $key_b => $link_b) {
				if ($key_a !== $key_b && $link_a === $link_b) {
					$error = __('リンク文字列は同じにできません。', 'affiliate-link-redirector');
					break 2;
				}
			}
		}
	}

	if (!isset($error)) {
		foreach (ALR_Setting::$settings_default as $key => $val) {
			if (!is_array($val)) {
				if (empty($_POST[$key])) {
					$post_data[$key] = false;
				} else if ("true" === $_POST[$key]) {
					$post_data[$key] = true;
				} else {
					$post_data[$key] = stripslashes(sanitize_text_field($_POST[$key]));
				}
			} else {
				foreach ($val as $asp => $data) {
					if (empty($_POST[$key][$asp])) {
						$post_data[$key][$asp] = false;
					} else if ("true" === $_POST[$key][$asp]) {
						$post_data[$key][$asp] = true;
					} else {
						$post_data[$key][$asp] = stripslashes(sanitize_text_field($_POST[$key][$asp]));
					}
				}
			}
		}

		ALR_Setting::update('settings', $post_data);

		$updated = __('設定を保存しました。', 'affiliate-link-redirector');
	}
}


if (isset($_POST['update_links'])) {

	if (!wp_verify_nonce($_POST[ALR_Link::NONCE_NAME], ALR_Link::NONCE_ACTION)) wp_die();

	if (!isset($error)) {

		ALR_Link::update_all_links();

		$updated = __('すべてのリンクデータを更新しました。', 'affiliate-link-redirector');
	}
}


?>
<div class="wrap">
	<h1>
		Affiliate Link Redirector
	</h1>
	<?php if (isset($error))  ALR_Menu::error_message($error); ?>
	<?php if (isset($updated))  ALR_Menu::updated_message($updated); ?>
	<form action="" method="post" accept-charset="UTF-8">
		<?php
		wp_nonce_field(ALR_Setting::NONCE_ACTION, ALR_Setting::NONCE_NAME);
		if ('301_redirect' === ALR_Setting::$settings['redirect_method']) {
			$redirect_method_checked['301_redirect'] = 'checked';
			$redirect_method_checked['302_redirect'] = '';
			$redirect_method_checked['307_redirect'] = '';
			$redirect_method_checked['javascript'] = '';
			$redirect_method_checked['meta_refresh'] = '';
		} else if ('302_redirect' === ALR_Setting::$settings['redirect_method']) {
			$redirect_method_checked['301_redirect'] = '';
			$redirect_method_checked['302_redirect'] = 'checked';
			$redirect_method_checked['307_redirect'] = '';
			$redirect_method_checked['javascript'] = '';
			$redirect_method_checked['meta_refresh'] = '';
		} else if ('307_redirect' === ALR_Setting::$settings['redirect_method']) {
			$redirect_method_checked['301_redirect'] = '';
			$redirect_method_checked['302_redirect'] = '';
			$redirect_method_checked['307_redirect'] = 'checked';
			$redirect_method_checked['javascript'] = '';
			$redirect_method_checked['meta_refresh'] = '';
		} else if ('javascript' === ALR_Setting::$settings['redirect_method']) {
			$redirect_method_checked['301_redirect'] = '';
			$redirect_method_checked['302_redirect'] = '';
			$redirect_method_checked['307_redirect'] = '';
			$redirect_method_checked['javascript'] = 'checked';
			$redirect_method_checked['meta_refresh'] = '';
		} else if ('meta_refresh' === ALR_Setting::$settings['redirect_method']) {
			$redirect_method_checked['301_redirect'] = '';
			$redirect_method_checked['302_redirect'] = '';
			$redirect_method_checked['307_redirect'] = '';
			$redirect_method_checked['javascript'] = '';
			$redirect_method_checked['meta_refresh'] = 'checked';
		}
		$robotstxt_disallow_disabled = ALR_IS_SUBDIR ? 'disabled' : '';
		$robotstxt_disallow_checked = ALR_Setting::$settings['robotstxt_disallow'] ? 'checked' : '';
		$pochipp_replace_checked = ALR_Setting::$settings['pochipp_replace'] ? 'checked' : '';

		$asp_checked['all'] = 'checked';
		foreach (ALR_Setting::$settings_default['asp_check'] as $asp => $check) {
			if (isset(ALR_Setting::$settings['asp_check'][$asp])) {
				$asp_checked[$asp] = (bool)ALR_Setting::$settings['asp_check'][$asp] ? 'checked' : '';
			} else {
				$asp_checked[$asp] = '';
			}
			if (empty(ALR_Setting::$settings['asp_check'][$asp])) $asp_checked['all'] = '';
		}
		foreach (ALR_Setting::$settings_default['asp_string'] as $asp => $link) {
			$asp_string[$asp] = isset(ALR_Setting::$settings['asp_string'][$asp]) ? ALR_Setting::$settings['asp_string'][$asp] : ALR_Setting::$settings_default['asp_string'][$asp];
		}
		$asp_string['all'] = isset(ALR_Setting::$settings['all_string']) ? ALR_Setting::$settings['all_string'] : ALR_Setting::$settings_default['all_string'];
		?>
		<input type="hidden" name="options_name" value="<?php echo esc_attr(ALR_Setting::OPTIONS_NAME['settings']); ?>">
		<h2 class="title">一般設定</h2>
		<table class="form-table" role="presentation">
			<tbody>
				<tr>
					<th scope="row">
						リダイレクト方法
					</th>
					<td colspan="2">
						<fieldset>
							<label><input type="radio" name="redirect_method" value="301_redirect" <?php echo esc_attr($redirect_method_checked['301_redirect']) ?>>301 redirect</label>
							<label><input type="radio" name="redirect_method" value="302_redirect" <?php echo esc_attr($redirect_method_checked['302_redirect']) ?>>302 redirect</label>
							<label><input type="radio" name="redirect_method" value="307_redirect" <?php echo esc_attr($redirect_method_checked['307_redirect']) ?>>307 redirect</label>
							<label><input type="radio" name="redirect_method" value="javascript" <?php echo esc_attr($redirect_method_checked['javascript']) ?>>javascript</label>
							<label><input type="radio" name="redirect_method" value="meta_refresh" <?php echo esc_attr($redirect_method_checked['meta_refresh']) ?>>meta refresh</label>
						</fieldset>
					</td>
				</tr>
				<tr>
					<th scope="row">
						robots.txt
					</th>
					<td>
						<input type="checkbox" name="robotstxt_disallow" id="robotstxt_disallow" value="true" <?php echo esc_attr($robotstxt_disallow_checked . ' ' . $robotstxt_disallow_disabled) ?>>
						<label for="robotstxt_disallow">リダイレクトURLをDisallow</label>
					</td>
					<td>
						※サブディレクトリでは利用できません。
						※自分でrobots.txtを設置している場合は適用されません。
					</td>
				</tr>
				<tr>
					<th scope="row">
						Pochipp
					</th>
					<td>
						<input type="checkbox" name="pochipp_replace" id="pochipp_replace" value="true" <?php echo esc_attr($pochipp_replace_checked) ?>>
						<label for="pochipp_replace">Pochippコードを変換</label>
					</td>
					<td>
						※Pochippコードを記事に書く場合はチェックを外してください。
					</td>
				</tr>
			</tbody>
		</table>
		<h2 class="title">リンク設定</h2>
		<table class="form-table" role="presentation">
			<tbody>
				<tr>
					<th scope="row">
						<label for="all_check">すべて</label>
					</th>
					<td>
						<input type="checkbox" name="all_check" id="all_check" value="true" <?php echo esc_attr($asp_checked['all']) ?>>
					</td>
					<td>
						<label for="all_link">リンク文字列</label>
						<input type="text" name="all_string" id="all_link" value="<?php echo esc_attr($asp_string['all']) ?>">
						※投稿などのURLスラッグと重複しないように注意してください。
					</td>
				</tr>
				<?php foreach (ALR_Setting::$settings_default['asp_string'] as $asp => $link) : ?>
					<tr>
						<th scope="row">
							<label for="<?php echo esc_attr($asp) ?>_check"><?php echo esc_attr(ALR_Setting::get_asp_names()[$asp]) ?></label>
						</th>
						<td>
							<input type="checkbox" name="asp_check[<?php echo esc_attr($asp) ?>]" id="<?php echo esc_attr($asp) ?>_check" value="true" <?php echo esc_attr($asp_checked[$asp]) ?>>
						</td>
						<td>
							<label for="<?php echo esc_attr($asp) ?>_link">リンク文字列</label>
							<input type="text" name="asp_string[<?php echo esc_attr($asp) ?>]" id="<?php echo esc_attr($asp) ?>_link" value="<?php echo esc_attr($asp_string[$asp]) ?>">
							例:<?php echo esc_url(home_url('/') . esc_attr($asp_string['all']) . '/' . esc_attr($asp_string[$asp]) . '/*****') ?>
						</td>
					</tr>
				<?php endforeach; ?>
			</tbody>
		</table>
		<p class="submit">
			<button type="submit" class="button button-primary"><?php echo esc_html__('変更を保存', 'affiliate-link-redirector') ?></button>
		</p>
	</form>
	<hr>
	<h2 class="title">リンクデータ更新</h2>
	<p>処理に時間がかかります。最初に一度だけ実行してください。</p>
	<form action="" method="post" accept-charset="UTF-8">
		<?php
		wp_nonce_field(ALR_Link::NONCE_ACTION, ALR_Link::NONCE_NAME);
		?>
		<input type="hidden" name="update_links" value="true">
		<p class="submit">
			<button type="submit" class="button button-secondary"><?php echo esc_html__('すべてのリンクデータを更新', 'affiliate-link-redirector') ?></button>
		</p>
	</form>
</div>
<script>
	document.addEventListener("DOMContentLoaded", function() {
		const all_check = document.getElementById('all_check');
		all_check.addEventListener('change', function() {
			const all_checked = this.checked;
			const asp_checks = document.querySelectorAll('[name*="asp_check"]');
			asp_checks.forEach(function(asp_check) {
				if (true === all_checked) {
					asp_check.checked = true;
				} else if (false === all_checked) {
					asp_check.checked = false;
				}
			});
		});
	});
</script>