<?php

if (!defined('ABSPATH')) exit;


class ALR_Link
{

	private function __construct()
	{
	}


	const DATABASE_NAME = [
		'links' => 'alr_links',
		'relationships' => 'alr_relationships',
	];

	public static $asp_patterns = null;

	const NONCE_ACTION = 'alr_link_nonce_action';
	const NONCE_NAME = 'alr_link_nonce_name';


	public static function init()
	{

		add_action('updated_postmeta', function () {
			global $post;

			if (isset($post->post_type) && ('post' === $post->post_type || 'page' === $post->post_type || (ALR_IS_SWELL && ('ad_tag' === $post->post_type || 'blog_parts' === $post->post_type)))) {
				//$post = get_post($post->ID);
				if (!empty($post)) {
					$links_new = self::search_links($post);
					$links_old = self::get_post_links($post->ID);
					self::update_table_links($post->ID, $links_new, $links_old);
				}
			}
		});

		add_action('before_delete_post', function () {
			global $post;

			if (isset($post->post_type) && ('post' === $post->post_type || 'page' === $post->post_type || (ALR_IS_SWELL && ('ad_tag' === $post->post_type || 'blog_parts' === $post->post_type)))) {
				$links_old = self::get_post_links($post->ID);
				if (!empty($links_old)) {
					foreach ($links_old as $link) {
						self::delete_table_link($post->ID, $link);
					}
				}
			}
		});


		self::set_asp_patterns();

		add_filter('the_content', function ($content) {
			if (!is_admin()) {
				global $post;

				if (isset($post->post_type) && ('post' === $post->post_type || 'page' === $post->post_type || (ALR_IS_SWELL && ('ad_tag' === $post->post_type || 'blog_parts' === $post->post_type)))) {
					$links = self::get_post_links($post->ID);

					if (!empty($links)) {
						foreach ($links as $link_from) {
							$link_from = htmlspecialchars($link_from);
							$link_to = esc_url(self::get_link_to($link_from));
							$content = str_replace($link_from, $link_to, $content);
						}
					}
				}

				if (ALR_Setting::$settings['pochipp_replace']) {
					$content = str_replace('class="pochipp-box', 'class="pochip-box', $content);
				}
			}

			return $content;
		});

		add_action('wp_enqueue_scripts', function () {
			if (!is_admin()) {
				if (ALR_Setting::$settings['pochipp_replace']) {
					wp_dequeue_style('pochipp-front');

					wp_register_style('pochip-front', false);
					wp_enqueue_style('pochip-front');
					ob_start();
					require_once(ABSPATH . '/wp-content/plugins/pochipp/dist/css/style.css');
					$content = ob_get_contents();
					ob_end_clean();
					$content = str_replace('.pochipp-box', '.pochip-box', $content);
					wp_add_inline_style('pochip-front', $content);
				}
			}
		}, 100);


		add_filter('do_shortcode_tag', function ($content, $tag, $attr) {
			if (!is_admin()) {

				if (ALR_IS_SWELL && ('ad_tag' === $tag || 'blog_parts' === $tag)) {
					$links = self::get_post_links($attr['id']);

					if (!empty($links)) {
						foreach ($links as $link_from) {
							//$link_from = htmlspecialchars($link_from);
							$link_to = esc_url(self::get_link_to($link_from));
							$content = str_replace($link_from, $link_to, $content);
						}
					}
				}
			}

			return $content;
		}, 10, 3);


		add_filter('template_redirect', function () {
			if (!is_admin()) {
				$url = get_pagenum_link();

				$links = self::get_table_links();

				if (!empty($links)) {
					foreach ($links as $link_from) {
						$link_to = self::get_link_to($link_from);
						if ($link_to === rtrim($url, '/')) {
							$link_from = filter_var($link_from, FILTER_SANITIZE_URL);
							if ('301_redirect' === ALR_Setting::$settings['redirect_method']) {
								wp_redirect($link_from, 301);
								exit;
							} else if ('302_redirect' === ALR_Setting::$settings['redirect_method']) {
								wp_redirect($link_from, 302);
								exit;
							} else if ('307_redirect' === ALR_Setting::$settings['redirect_method']) {
								wp_redirect($link_from, 307);
								exit;
							} else if ('javascript' === ALR_Setting::$settings['redirect_method']) {
								status_header(200);
?>
								<script>
									window.location.href = "<?php echo filter_var($link_from, FILTER_SANITIZE_URL) ?>";
								</script>
							<?php
								exit;
							} else if ('meta_refresh' === ALR_Setting::$settings['redirect_method']) {
								status_header(200);
							?>

								<head>
									<meta http-equiv="refresh" content="0;URL=<?php echo filter_var($link_from, FILTER_SANITIZE_URL) ?>">
								</head>

								<body>
								</body>
<?php
								exit;
							}
						}
					}
				}
			}
		});


		add_filter('robots_txt', function ($output) {
			if (!is_admin()) {
				if (!ALR_IS_SUBDIR) {
					if (ALR_Setting::$settings['robotstxt_disallow']) {
						// double("") not single('')
						$output .= "\nUser-agent: *\nDisallow: /" . ALR_Setting::$settings['all_string'] . "/\n";
					}
				}
			}

			return $output;
		});
	}


	private static function set_asp_patterns()
	{
		self::$asp_patterns = [
			'amazon' => [
				[
					'domain' => '//www.amazon.co.jp/',
					'id' => '-22',
					// 'start' => 'dp/',
					// 'end' => '',
				],
				[
					'domain' => '//amzn.to/',
					'id' => '',
					// 'start' => '',
					// 'end' => '',
				],
			],
			'rakuten' => [
				[
					'domain' => '//hb.afl.rakuten.co.jp/',
					'id' => '',
					// 'start' => '',
					// 'end' => '',
				],
				[
					'domain' => '//product.rakuten.co.jp/',
					'id' => 'sc2id=',
					// 'start' => '',
					// 'end' => '',
				],
			],
			'a8net' => [
				[
					'domain' => '//px.a8.net/',
					'id' => '',
					// 'start' => 'ejp?a8mat=',
					// 'end' => '',
				],
			],
			'moshimo' => [
				[
					'domain' => '//af.moshimo.com/',
					'id' => '',
					// 'start' => 'click?',
					// 'end' => '',
				],
				[
					'domain' => '//msm.to/',
					'id' => '',
					// 'start' => '',
					// 'end' => '',
				],
			],
			'rentracks' => [
				[
					'domain' => '//www.rentracks.jp/',
					'id=' => '',
					// 'start' => 'r.html?',
					// 'end' => '',
				],
			],
			'infotop' => [
				[
					'domain' => '//www.infotop.jp/',
					'id=' => '',
					// 'start' => 'click.php?',
					// 'end' => '',
				],
			],
		];
	}


	private static function check_link_match($link)
	{
		foreach (self::$asp_patterns as $asp => $patterns) {
			// 			if (empty(ALR_Setting::$settings['asp_check'][$asp]) || true !== ALR_Setting::$settings['asp_check'][$asp]) {
			// 				continue;
			// 			}
			foreach ($patterns as $pattern) {
				if (false !== stripos($link, $pattern['domain'])) {
					if (empty($pattern['id']) || false !== stripos($link, $pattern['id'])) {
						return true;
					}
				}
			}
		}

		return false;
	}


	private static function get_link_to($link)
	{
		foreach (self::$asp_patterns as $asp => $patterns) {
			if (empty(ALR_Setting::$settings['asp_check'][$asp]) || true !== ALR_Setting::$settings['asp_check'][$asp]) {
				continue;
			}
			foreach ($patterns as $pattern) {
				if (false !== stripos($link, $pattern['domain'])) {
					if (empty($pattern['id']) || false !== stripos($link, $pattern['id'])) {
						// if (!empty($pattern['start'])) {
						// 	if (($start = stripos($link, $pattern['start'])) !== false) {
						// 		$link = substr($link, $start + strlen($pattern['start']));
						// 	}
						// } else {
						// 	if (($start = stripos($link, $pattern['domain'])) !== false) {
						// 		$link = substr($link, $start + strlen($pattern['domain']));
						// 	}
						// }
						// if (!empty($pattern['end'])) {
						// 	if (($end = stripos($link, $pattern['end'])) !== false) {
						// 		$link = substr($link, 0, $end);
						// 	}
						// }
						$link = htmlspecialchars_decode($link);
						// $link = str_replace(['&', '=', '+', '%2F', '%3A'], '', $link);
						$link = md5($link);
						$link = home_url('/') . ALR_Setting::$settings['all_string'] . '/' . ALR_Setting::$settings['asp_string'][$asp] . '/' . $link;
					}
				}
			}
		}

		return $link;
	}


	private static function get_post_links($post_id)
	{

		global $wpdb;

		$table_name['links'] = $wpdb->prefix . self::DATABASE_NAME['links'];
		$table_name['relationships'] = $wpdb->prefix . self::DATABASE_NAME['relationships'];

		// double not single('')
		$join = ' INNER JOIN ' . $table_name['relationships'] . ' ON (' . $table_name['links'] . '.link_id = ' . $table_name['relationships'] . '.link_id) ';
		$links = $wpdb->get_col($wpdb->prepare('SELECT link_url FROM ' . $table_name['links'] . $join . 'WHERE object_id = %d', $post_id));

		return $links;
	}


	private static function get_table_links()
	{

		global $wpdb;

		$table_name = $wpdb->prefix . self::DATABASE_NAME['links'];

		$links = get_transient(ALR_Setting::TRANSIENT_NAME['table_links']);
		if (empty($links)) {
			$links = $wpdb->get_col($wpdb->prepare('SELECT link_url FROM ' . $table_name));
			set_transient(ALR_Setting::TRANSIENT_NAME['table_links'], $links, 12 * HOUR_IN_SECONDS);
		}

		return $links;
	}


	private static function update_table_links($post_id, $links_new, $links_old)
	{

		$links_new = !empty($links_new) ? $links_new : [];
		$links_old = !empty($links_old) ? $links_old : [];

		$links_add = array_diff($links_new, $links_old);
		$links_delete = array_diff($links_old, $links_new);

		if (!empty($links_add)) {
			foreach ($links_add as $link) {
				self::add_table_link($post_id, $link);
			}
		}

		if (!empty($links_delete)) {
			foreach ($links_delete as $link) {
				self::delete_table_link($post_id, $link);
			}
		}

		delete_transient(ALR_Setting::TRANSIENT_NAME['table_links']);
	}


	private static function add_table_link($post_id, $link)
	{

		global $wpdb;

		$table_name = $wpdb->prefix . self::DATABASE_NAME['links'];

		$link_id = $wpdb->get_var($wpdb->prepare('SELECT link_id FROM ' . $table_name . ' WHERE link_url = %s', $link));

		if (empty($link_id)) {

			$table_data = [
				'link_url' => filter_var($link, FILTER_SANITIZE_URL),
			];
			$format = [
				'%s',
			];

			$wpdb->insert(
				$table_name,
				$table_data,
				$format,
			);
			$link_id = $wpdb->insert_id;
		}


		$table_name = $wpdb->prefix . self::DATABASE_NAME['relationships'];

		$link_row = $wpdb->get_row($wpdb->prepare('SELECT * FROM ' . $table_name . ' WHERE object_id = %d AND link_id = %d', $post_id, $link_id), ARRAY_A);

		if (empty($link_row)) {

			$table_data = [
				'object_id' => (int)$post_id,
				'link_id' => (int)$link_id,
			];
			$format = [
				'%s',
				'%s',
			];

			$wpdb->insert(
				$table_name,
				$table_data,
				$format,
			);
		}
	}


	private static function delete_table_link($post_id, $link)
	{
		global $wpdb;

		$table_name = $wpdb->prefix . self::DATABASE_NAME['links'];

		$link_id = $wpdb->get_var($wpdb->prepare('SELECT link_id FROM ' . $table_name . ' WHERE link_url = %s', $link));

		if (!empty($link_id)) {

			$target_table = [
				'link_id' => (int)$link_id,
			];
			$target_format = [
				'%d',
			];

			$wpdb->delete(
				$table_name,
				$target_table,
				$target_format,
			);
		}


		$table_name = $wpdb->prefix . self::DATABASE_NAME['relationships'];

		$link_row = $wpdb->get_row($wpdb->prepare('SELECT * FROM ' . $table_name . ' WHERE object_id = %d AND link_id = %d', $post_id, $link_id), ARRAY_A);

		if (!empty($link_row)) {

			$target_table = [
				'object_id' => (int)$post_id,
				'link_id' => (int)$link_id,
			];
			$target_format = [
				'%d',
				'%d',
			];

			$wpdb->delete(
				$table_name,
				$target_table,
				$target_format,
			);
		}
	}


	public static function update_all_links()
	{

		@set_time_limit(300);

		$post_type = ['post', 'page'];
		if (ALR_IS_SWELL) {
			$post_type = array_merge($post_type, ['ad_tag', 'blog_parts']);
		}

		$arg = [
			'numberposts' => -1,
			'post_type' => $post_type,
			'post_status' => 'publish',
		];
		$posts = get_posts($arg);

		if (!empty($posts)) {
			foreach ($posts as $post) {
				$post = get_post($post->ID);
				if (!empty($post)) {
					$links_new = self::search_links($post);
					$links_old = self::get_post_links($post->ID);
					self::update_table_links($post->ID, $links_new, $links_old);
				}
			}
		}
	}


	private static function search_links($post)
	{

		if (ALR_IS_SWELL) {
			if ('ad_tag' === $post->post_type) {
				$html = get_post_meta($post->ID, 'ad_img', true);
			} else if ('blog_parts' === $post->post_type) {
				$html = $post->post_content;
			} else {
				$html = $post->post_content;
			}
		} else {
			$html = $post->post_content;
		}

		$links = [];

		if (!empty($html)) {
			$dom = new DOMDocument;
			@$dom->loadHTML($html);
			$tags = $dom->getElementsByTagName('a');
			if (!empty($tags)) {
				foreach ($tags as $tag) {
					$link = $tag->getAttribute('href');
					if (false !== strpos($link, '//')) {
						if (self::check_link_match($link)) {
							if (!empty($link)) {
								$links[] = filter_var($link, FILTER_SANITIZE_URL); // escape
							}
						}
					}
				}
			}
		}

		if (!empty($links)) {
			$links = array_unique($links);
			usort($links, function ($a, $b) {
				return strlen($b) - strlen($a);
			});
			return $links;
		} else {
			return false;
		}
	}
}
