<?php

if (!defined('ABSPATH')) exit;


class ALR_Setting
{

	private function __construct()
	{
	}


	const OPTIONS_NAME = [
		'version' => 'alr_version',
		'settings' => 'alr_settings',
	];

	const TRANSIENT_NAME = [
		'table_links' => 'alr_table_links',
	];

	const NONCE_ACTION = 'alr_setting_nonce_action';
	const NONCE_NAME = 'alr_setting_nonce_name';


	public static $settings_default = [];

	public static $settings = null;


	public static function init()
	{

		self::set_settings_default();
		self::set_settings();


		$version = get_option(self::OPTIONS_NAME['version']);

		if (empty($version)) {

			// install
			update_option(self::OPTIONS_NAME['version'], ALR_PLUGIN_VERSION);
			update_option(self::OPTIONS_NAME['settings'], self::$settings_default);
		} else if (ALR_PLUGIN_VERSION !== $version) {

			// version update
			update_option(self::OPTIONS_NAME['version'], ALR_PLUGIN_VERSION);
		}
	}


	private static function set_settings_default()
	{
		self::$settings_default = [
			'redirect_method' => '302_redirect',
			'robotstxt_disallow' => false,
			'pochipp_replace' => false,
			'all_string' => 'redirect',
			'asp_check' => [
				'amazon' => false,
				'rakuten' => false,
				'a8net' => false,
				'moshimo' => false,
				'rentracks' => false,
				'infotop' => false,
			],
			'asp_string' => [
				'amazon' => 'amazon',
				'rakuten' => 'rakuten',
				'a8net' => 'a8net',
				'moshimo' => 'moshimo',
				'rentracks' => 'rentracks',
				'infotop' => 'infotop',
			],
		];
	}

	private static function set_settings()
	{
		$settings = get_option(self::OPTIONS_NAME['settings']);
		$settings = !empty($settings) ? $settings : [];
		$settings = array_merge(self::$settings_default, $settings);
		self::$settings = $settings;
	}

	public static function get_asp_names()
	{
		return [
			'amazon' => 'Amazonアソシエイト',
			'rakuten' => '楽天アフィリエイト',
			'yahoo' => 'Yahoo!アフィリエイト',
			'a8net' => 'A8.net',
			'moshimo' => 'もしもアフィリエイト',
			'rentracks' => 'レントラックス',
			'infotop' => 'インフォトップ',
		];
	}


	public static function update($options_key, $data)
	{
		if ('settings' === $options_key) {
			update_option(self::OPTIONS_NAME[$options_key], $data);
			self::$settings = $data ? $data : self::$settings_default;
		} else {
			return;
		}
	}
}
